#########################################################
## This script calculates the PRES (Patent Retrieval   ##
## Evaluation Score) for search results in TREC format ##
## In addition, it calcualtes the MAP and Avg.Recall   ##
## too. Script can print result per topic.             ##
## Default Nmax = 1000                                 ##
## Usage: perl PRES.pl [-d] qrels results.file [Nmax]  ##
## Copyright: 27-6-2009                                ##
## Walid Magdy - DCU                                   ##
## Last modified: 6-7-2010                             ##
#########################################################

$usage = "Usage: perl PRESeval.pl [-d:(for details)] qrels results [Nmax] > Score\n";

if($ARGV[0] eq "-d"){
  $arg = 1;
  $details = 1;
}
$qrels = $ARGV[$arg++] or die $usage;
$results = $ARGV[$arg++] or die $usage;
$Nmax  = $ARGV[$arg++] or $Nmax = 1000;
$total = 0;

## Reading qrels file (assuming relevant documents are only in qrels)
open(IN, $qrels) or die "Can't find $qrels\n$usage";
while(<IN>){
  if(/(\S+)\s+Q*0*\s*(\S+)\s+(\d+)/){
    if($3){
      $reldocs{lc($1)}{lc($2)}++;
      $nrel{lc($1)}++;
    }
  }
}
close(IN);

## Reading the results file that is in TREC format.
## For different formats, please change the regular expression.
open(IN,$results) or die "Can't find $results\n$usage";
do{
  $result = <IN>;
  if($result =~ /(\S+)\s+Q0\s+(\S+)\s+(\d+)/ || $result =~ /^$/)
  {
    $q = lc $1;
    $doc = lc $2;
    $r++;
    if(exists $nrel{$prev} && $q ne $prev){
      if($found){
	$Ncollection = $Nmax+$nrel{$prev};
	$remain = $nrel{$prev}-$found;
	$SumRank += $remain*($Ncollection-($remain-1)/2);
	$PRES{$prev} = 1 - ($SumRank-$nrel{$prev}*($nrel{$prev}+1)/2)/($nrel{$prev}*($Ncollection-$nrel{$prev}));
      }
      else{
	$PRES{$prev}=0;
      }
      $MPRES += $PRES{$prev};
      $total++;
      $rlist =~ s/\s+$//;
      $rlist =~ s/\s+/,/g;
      $AP{$prev}/=$nrel{$prev};
      $MAP += $AP{$prev};
      $recall{$prev} = $found/$nrel{$prev};
      $avgRecall += $found/$nrel{$prev};
      $details{$prev}="Ret/Rel=$found/$nrel{$prev}, Rel_Ranks={$rlist}";
      $found = 0;
      $SumRank = 0;
      $r=1;
      $rlist="";
    }
    elsif($q ne $prev){
      $found = 0;
      $r = 1;
    }
    if($r<=$Nmax && exists $reldocs{$q}{$doc}){
      $SumRank += $r;
      $found++;
      $rlist.="$r ";
      $AP{$q}+=$found/$r;
    }
    $prev = $q;
  }
}while($result);

print "File name: $results\n\n";

if($details){
  for $q (sort {$a <=> $b} keys %PRES){
    print "$q\t$PRES{$q}\t$AP{$q}\t$recall{$q}\t($details{$q})\n";
  }
  print "\n\n";
}

print "Total Queries that has retrieved results: $total\n";
print "PRES = ",$MPRES/$total,"\n";
print "MAP = ",$MAP/$total,"\n";
print "Avg Recall = ",$avgRecall/$total,"\n\n";

print "Total Queries that has judgements: ",keys(%nrel)+0,"\n";
print "avg PRES/all topics = ",$MPRES/keys(%nrel),"\n";
print "MAP = ",$MAP/keys(%nrel),"\n";
print "Avg Recall = ",$avgRecall/keys(%nrel),"\n\n";

print "Script Provided by School of Computing - DCU\n\n";
